<?php
if (!defined('_PS_VERSION_')) {
    exit;
}
/**
 * Contains methods that can be used to link gateway with database data
 *
 *
 * @version 1.0
 * @author nalic
 */
class Database
{
    //Properties
	protected $db;
	//table prefix if any
	protected $table_prefix;

    protected $transactions_table_name;

    protected $transaction_adjustments_table_name;

    protected $database_name;

	public function __construct($table_prefix, $host, $db_name, $user, $pass){
		try {
            if(empty($table_prefix) ||strlen($table_prefix) == 0){
                $this->throwDbError('Table prefix must be set!');
            }
			$this->db           = new PDO('mysql:host=' . $host . ';dbname=' . $db_name, $user, $pass);
            $this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_WARNING);
			$this->table_prefix = $table_prefix;

            $this->transactions_table_name = $this->table_prefix . "gpndata_transactions";
            $this->transaction_adjustments_table_name = $this->table_prefix . "gpndata_transaction_adjustments";
            $this->database_name = $db_name;
		}
		catch(PDOException $e){
			// If something goes wrong, PDO throws an exception with a nice error message.
            $this->throwDbError($e->getMessage());
		}
	}

	public function createTables(){

		try {

            $this->db->query(
                "
				CREATE TABLE IF NOT EXISTS `" . $this->transactions_table_name . "` (
					  Id BIGINT(20) NOT NULL AUTO_INCREMENT,
                      order_id CHAR(36) NULL,
					  merchant_trans_id CHAR(50) NOT NULL,
                      gate_trans_id CHAR(36) NULL,
					  transaction_ref CHAR(50) DEFAULT NULL,
					  curcode VARCHAR(3) NOT NULL,
					  payment_amount DECIMAL(10, 2) NOT NULL,
					  merchantspecific1 VARCHAR(200) DEFAULT NULL,
					  merchantspecific2 VARCHAR(200) DEFAULT NULL,
					  merchantspecific3 VARCHAR(200) DEFAULT NULL,
					  merchantspecific4 VARCHAR(200) DEFAULT NULL,
                      merchantspecific5 VARCHAR(200) DEFAULT NULL,
                      three_d_md VARCHAR (36) DEFAULT NULL,
					  is_captured TINYINT DEFAULT 0,
					  is_voided TINYINT DEFAULT 0,
					  is_authorized TINYINT DEFAULT 0,
					  is_declined TINYINT DEFAULT 0,
                      is_refunded TINYINT DEFAULT 0,
                      is_chargeback TINYINT DEFAULT 0,
                      is_refund_pending TINYINT DEFAULT 0,
                      is_payment_failed TINYINT DEFAULT 0,
                      payment_failure_description VARCHAR(255) DEFAULT NULL,
                      is_notification_pending TINYINT DEFAULT 0,
                      installment_count TINYINT DEFAULT NULL,
                      customer_token VARCHAR(36),
					  date_added DATETIME DEFAULT NULL,
					  date_modified DATETIME DEFAULT NULL,
					  PRIMARY KEY (Id)
				) ENGINE=INNODB DEFAULT COLLATE=utf8_general_ci;"
        );

			$this->db->query(
					"
			CREATE TABLE IF NOT EXISTS `" . $this->transaction_adjustments_table_name . "` (
				  transaction_adjustment_id BIGINT(20) NOT NULL AUTO_INCREMENT,
				  gate_trans_id NVARCHAR(36) NOT NULL,
                  amount DECIMAL(10, 2) NOT NULL,
				  status VARCHAR(50) NOT NULL,
				  state VARCHAR(50) NOT NULL,
				  description VARCHAR(255) DEFAULT NULL,
				  additional_text VARCHAR(255) DEFAULT NULL,
				  errormessage VARCHAR(255) DEFAULT NULL,
				  date_added DATETIME DEFAULT NULL,
				  PRIMARY KEY (transaction_adjustment_id)
			) ENGINE=INNODB DEFAULT COLLATE=utf8_general_ci;"
			);

            //$query = "ALTER TABLE ".$this->database_name . "." . $this->transaction_adjustments_table_name . " ADD CONSTRAINT FK_transaction_adjustments_gpn_gate_trans_Id FOREIGN KEY (gate_trans_id) REFERENCES ".
            //    $this->database_name .".". $this->transactions_table_name ."(gate_trans_id) ON DELETE RESTRICT ON UPDATE RESTRICT";

            //$this->db->query($query);

		}
		catch(Exception $e){
			echo $e->getMessage();
			return false;
		}

		return true;
	}

	public function dropTables(){
		try {

            $this->db->query("DROP TABLE IF EXISTS `" . $this->transaction_adjustments_table_name . "`;");
			$this->db->query("DROP TABLE IF EXISTS `" . $this->transactions_table_name . "`;");
		}
		catch(Exception $exception){
			$this->throwDbError($exception->getMessage());
			return false;
		}
		return true;
	}

	/*Mandatory fields simple example. Extened the array with additional table fields if there is a need
	$orderData = array('order_id'=>'','merchant_trans_id'=>'', 'gate_trans_id'=>'', 'date_added' => 'now()', 'curcode' => '', 'total_amount' => ''); */
	public function addTransaction($transaction){

		$query = "INSERT INTO `" . $this->transactions_table_name . "` SET " . $this->bindFields($transaction) . ", date_added = now(), is_notification_pending = 1";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($transaction) or $this->throwDbError($stmt->errorInfo());
	}

    public function addTransactionIfDoesNotExist($transaction){

        if(!$this->transactionExistsByMerchantTransactionId($transaction['merchant_trans_id'])){
            $query = "INSERT INTO `" . $this->transactions_table_name . "` SET " . $this->bindFields($transaction) . ", date_added = now()";
            $stmt  = $this->db->prepare($query);
            $stmt->execute($transaction) or $this->throwDbError($stmt->errorInfo());
        }
	}

    public function isNotificationPending($merchant_trans_id){

        $stmt = $this->db->prepare("SELECT * FROM `" . $this->transactions_table_name . "` WHERE `merchant_trans_id` = :merchant_trans_id AND is_notification_pending = 1 LIMIT 1");
		$stmt->bindParam(':merchant_trans_id', $merchant_trans_id, PDO::PARAM_STR);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());

		return $stmt->rowCount() > 0;
    }

    public function updateTransactionByTransactionId($transaction){

		$query = "UPDATE `" .
		        $this->transactions_table_name .
		         "` SET " .
		         $this->bindFields($transaction, 'Id') .
		         ", date_modified = now() " .
		         " WHERE `Id` = :Id";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($transaction) or $this->throwDbError($stmt->errorInfo());
        return $stmt->rowCount() > 0;
	}

	public function updateTransaction($transaction){

		$query = "UPDATE `" .
		        $this->transactions_table_name .
		         "` SET " .
		         $this->bindFields($transaction, 'gate_trans_id') .
		         ", date_modified = now() " .
		         " WHERE `gate_trans_id` = :gate_trans_id";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($transaction) or $this->throwDbError($stmt->errorInfo());
        return $stmt->rowCount() > 0;
	}

    public function updateTransactionByMerchantTransactionId($transaction){

		$query = "UPDATE `" .
		        $this->transactions_table_name .
		         "` SET " .
		         $this->bindFields($transaction, 'merchant_trans_id') .
		         ", date_modified = now() " .
		         " WHERE `merchant_trans_id` = :merchant_trans_id";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($transaction) or $this->throwDbError($stmt->errorInfo());
	}

	public function getTransaction($transaction_id, $fetch_transaction_adjustments = false){

		$stmt = $this->db->prepare("SELECT * FROM `" . $this->transactions_table_name . "` WHERE `gate_trans_id` = :gate_trans_id LIMIT 1");
		$stmt->bindParam(':gate_trans_id', $transaction_id, PDO::PARAM_INT);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());
		if($stmt->rowCount() > 0){
			$order = $this->getData($stmt);
			if($fetch_transaction_adjustments){
				$order['transactions'] = $this->getTransactions($order['transaction_id']);
			}

			return $order;
		}
		else{
			return false;
		}
	}

    public function getTransactionByOrderId($order_id, $fetch_transaction_adjustments = false){
        $data = array();
		$stmt = $this->db->prepare("SELECT * FROM `" . $this->transactions_table_name . "` WHERE `order_id` = :order_id LIMIT 1");
		$stmt->bindParam(':order_id', $order_id, PDO::PARAM_STR);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());
		if($stmt->rowCount() > 0){
			$transaction = $this->getData($stmt);
            $data['transaction'] = $transaction;
			if($fetch_transaction_adjustments){
				$data['transaction_adjustments'] = $this->getTransactionAdjustments($transaction['gate_trans_id']);
			}
			return $data;
		}
		else{
			return false;
		}
	}

	public function getTransactionByMerchantTransactionId($merchant_transaction_id, $fetch_transaction_adjustments = false){
        $data = array();
		$stmt = $this->db->prepare("SELECT * FROM `" . $this->transactions_table_name . "` WHERE `merchant_trans_id` = :merchant_trans_id LIMIT 1");
		$stmt->bindParam(':merchant_trans_id', $merchant_transaction_id, PDO::PARAM_STR);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());
		if($stmt->rowCount() > 0){
			$transaction = $this->getData($stmt);
            $data['transaction'] = $transaction;
			if($fetch_transaction_adjustments){
				$data['transaction_adjustments'] = $this->getTransactionAdjustments($transaction['gate_trans_id']);
			}
			return $data;
		}
		else{
			return false;
		}
	}

	public function transactionExistsByMerchantTransactionId($order_id){

		$stmt = $this->db->prepare("SELECT * FROM `" . $this->transactions_table_name . "` WHERE `merchant_trans_id` = :merchant_trans_id LIMIT 1");
		$stmt->bindParam(':merchant_trans_id', $order_id, PDO::PARAM_INT);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());

		return $stmt->rowCount() > 0;
	}

	//Mandatory fields simple example. Extened the array with additional table fields if there is a need
	//$orderData = array('gpndata_order_id'=>'','status'=>'', 'state'=>'', 'description' => '', 'date_added' => 'now()', 'amount' => '');
	public function addTransactionAdjustmentFromArray($adjustment){
        if($this->isTransactionAdjustmentDataInsertNeeded($adjustment['gate_trans_id'])){
            $query = "INSERT INTO `" . $this->transaction_adjustments_table_name . "` SET " . $this->bindFields($adjustment) . ", date_added = now()";
            $stmt  = $this->db->prepare($query);
            $stmt->execute($adjustment) or $this->throwDbError($stmt->errorInfo());
        }
	}

	public function addTransactionAdjustment($gate_transaction_id,  $amount, $status, $state, $description = '', $additional_text='', $errormessage = ''){

		$query = "INSERT INTO `" .$this->database_name.$this->transaction_adjustments_table_name .
                "` SET gate_trans_id=:gate_trans_id, amount=:amount, status=:status, state=:state, description=:description, additional_text=:additional_text, errormessage=:errormessage, date_added = now()";
        $stmt  = $this->db->prepare($query);

        $stmt->bindParam(':gate_trans_id', $gate_transaction_id, PDO::PARAM_STR);
        $stmt->bindParam(':amount', $amount, PDO::PARAM_STR);
        $stmt->bindParam(':status', $status, PDO::PARAM_STR);
        $stmt->bindParam(':state', $state, PDO::PARAM_STR);
        $stmt->bindParam(':description', $description, PDO::PARAM_STR);
        $stmt->bindParam(':additional_text', $additional_text, PDO::PARAM_STR);
        $stmt->bindParam(':errormessage', $errormessage, PDO::PARAM_STR);
        ///$stmt->execute() or $this->throwDbError($stmt->errorInfo());

        return $stmt->rowCount() > 0 or $this->throwDbError($stmt->errorInfo());

	}

	public function updateTransactionAdjustment($orderTransaction){

		$query = "UPDATE `" .
		         $this->transaction_adjustments_table_name .
		         "` SET " .
		         $this->bindFields($orderTransaction, 'transaction_id') .
		         " WHERE `transaction_id` = :transaction_id";
		$stmt  = $this->db->prepare($query);
		$stmt->execute($orderTransaction) or $this->throwDbError($stmt->errorInfo());
	}

	private function getTransactionAdjustments($gate_trans_id){

		$stmt = $this->db->prepare("SELECT * FROM `" . $this->transaction_adjustments_table_name . "` WHERE `gate_trans_id` = :gate_trans_id");
		$stmt->bindParam(":gate_trans_id", $gate_trans_id, PDO::PARAM_STR);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());
		if($stmt->rowCount() > 0){
			return $this->getData($stmt, true);
		}
		else{
            $err = $stmt->errorInfo();
			return false;
		}
	}

	public function getLastTransactionAdjustment($gate_trans_id){
		$stmt = $this->db->prepare(
				"SELECT * FROM `" . $this->transaction_adjustments_table_name . "` WHERE `gate_trans_id` = :gate_trans_id ORDER BY transaction_adjustment_id DESC LIMIT 1"
		);
        $stmt->bindParam(":gate_trans_id", $gate_trans_id, PDO::PARAM_STR);
		$stmt->execute() or $this->throwDbError($stmt->errorInfo());
		if($stmt->rowCount() > 0){
            return $this->getData($stmt)['status'];
		}
		else{
			return false;
		}
    }

	public function createUniqueKey($order_id){

		$order = $this->getOrder($order_id, false);

		return sha1($order_id . $order['gate_trans_id'] . $order['date_added']);
	}

	function bindFields($fields, $ignore_field = ''){
		end($fields);
		$lastField  = key($fields);
		$bindString = ' ';
		foreach($fields as $field => $data){

			if(strlen($ignore_field) > 0){
				if(strtolower($field) == strtolower($ignore_field)){
					continue;
				}
			}

			$bindString .= $field . '=:' . $field;
			$bindString .= ($field === $lastField ? ' ' : ',');
		}

		return $bindString;
	}

	private function getData($stmt, $fetchAll = false){

		if($fetchAll){
			$row = $stmt->fetchAll(PDO::FETCH_ASSOC);
			$stmt->closeCursor();
		}
		else{
			$row = $stmt->fetch(PDO::FETCH_ASSOC);
		}

		return $row;
	}

	private function throwDbError($errorInfo){
		$msg = print_r($errorInfo, true);
		throw new Exception($msg);
	}

	private function isTransactionAdjustmentDataInsertNeeded($gate_transaction_id){

		try {
			$transaction = $this->getLastTransactionAdjustment($gate_transaction_id);
			if($transaction['status'] === TransactionStatus::CANCELED || $transaction['status']
                === TransactionStatus::REFUND_PENDING || $transaction['status'] === TransactionStatus::REQUEST_PENDING){
				return false;
			}
		}
		catch(Exception $exception){
			echo 'Error occured while trying to check if data insert is needed. Error details: ' + $exception->getMessage();

			return false;
		}

		return true;
	}

	public function setDatabase($db){
		$this->db = $db;
	}
}
