<?php
/*
 *  @author PrestaShop SA <contact@prestashop.com>
 *  @copyright  2007-2015 PrestaShop SA
 *  @version  Release: $Revision: 1.2 $
 *
 *  International Registered Trademark & Property of PrestaShop SA
 */

/* Security */
if (!defined('_PS_VERSION_'))
	exit;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
class GpnData extends PaymentModule
{
	public $_API;
    public $db;

	public function __construct()
	{
		$this->name = 'gpndata';
		$this->author = 'GPN DATA Dev Team';
		$this->tab = 'payments_gateways';
		$this->version = '1.0.2';
		$this->is_eu_compatible = 1;
        $this->display = true;
        $this->controllers = array('payment', 'validation', 'ordervalidate');
        $this->bootstrap = true;

		$this->displayName = $this->l('GPN DATA International');
		$this->description = $this->l('Accept Credit card payments with GPN DATA International');
        $this->ps_versions_compliancy = array('min' => '1.7.0.0', 'max' => _PS_VERSION_);
		/* Check if cURL is enabled */
		if (!is_callable('curl_exec')){
			$this->warning = $this->l('cURL extension must be enabled on your server to use this module.');
			$this->active = false;
		}

		if (!Configuration::get('PS_SSL_ENABLED')){
            $this->warning = $this->l('Please enable SSL on your server to use full features of this module.');
		}
        $this->_include();
		$this->_load();
        parent::__construct();

	}

	private function _load(){
        try
        {
            if (!isset($this->_API)) {
                $this->_API = new GPNDataAPI(Configuration::get('GPNDATA_API_USER'),
                    Configuration::get('GPNDATA_API_PASSWORD'),
                    Configuration::get('GPNDATA_API_KEY'));

                $this->_API->transactionUrl = Configuration::get('GPNDATA_API_URL');
                $this->_API->prettyName = Configuration::get('PRETTY_NAME');
                $this->_API->iframeUrl = Configuration::get('IFRAME_URL');
                $this->_API->logoUrl = Configuration::get('LOGO_URL');
            }
            if(!isset($this->db)){
                $this->db = new Database(_DB_PREFIX_, _DB_SERVER_, _DB_NAME_, _DB_USER_, _DB_PASSWD_);
            }
        }
        catch (Exception $exception){
            echo $exception->getMessage();
			return false;
        }
        return true;
    }

    private function _include(){

        require_once(dirname(__FILE__) . '/library/gpndata.php');
        require_once(dirname(__FILE__) . '/class/database.php');
        require_once(dirname(__FILE__) . '/class/order_status.php');

    }

	public function install()
	{
        if(!isset($this->db)){
            $this->db = new Database(_DB_PREFIX_, _DB_SERVER_, _DB_NAME_, _DB_USER_, _DB_PASSWD_);
        }

        GpnDataOrderStatus::addOrderState(GpnDataOrderStatus::$DECLINED['name'], GpnDataOrderStatus::$DECLINED['value'], false, $this->name, false, '#b74800', false, false, false);
        GpnDataOrderStatus::addOrderState(GpnDataOrderStatus::$CHARGEBACK['name'], GpnDataOrderStatus::$CHARGEBACK['value'], true, $this->name, false, '#ec2e15', false, false, false);
        GpnDataOrderStatus::addOrderState(GpnDataOrderStatus::$PARTIAL_CAPTURED['name'], GpnDataOrderStatus::$PARTIAL_CAPTURED['value'], true, $this->name, true, '#00bb9d', true, true, true, 'payment');
        GpnDataOrderStatus::addOrderState(GpnDataOrderStatus::$PARTIAL_REFUNDED['name'], GpnDataOrderStatus::$PARTIAL_REFUNDED['value'], true, $this->name, false, '#ff9cb9', false, false, false);
        GpnDataOrderStatus::addOrderState(GpnDataOrderStatus::$PENDING_STAFF_APPROVAL['name'], GpnDataOrderStatus::$PENDING_STAFF_APPROVAL['value'], false, $this->name, false, '#FF8C00', false, false, false);

		return parent::install() && $this->db->createTables()
		&& $this->registerHook('payment')
		&& $this->registerHook('paymentReturn')
		&& $this->registerHook('adminPayment')
		&& $this->registerHook('adminOrder')
		&& $this->registerHook('paymentOptions')
		&& $this->registerHook('displayPayment')
		&& $this->registerHook('displayPaymentTop');
	}

	public function uninstall()
	{
        Configuration::deleteByName('GPNDATA_API_KEY');
        Configuration::deleteByName('GPNDATA_API_USER');
        Configuration::deleteByName('GPNDATA_API_PASSWORD');
        Configuration::deleteByName('GPNDATA_API_URL');
        Configuration::deleteByName('GPNDATA_PRETTY_NAME');
        Configuration::deleteByName('GPNDATA_IFRAME_URL');
        Configuration::deleteByName('GPNDATA_LOGO_URL');
        Configuration::deleteByName('GPNDATA_PAYMENT_MERCHANT_NAME');
        Configuration::deleteByName('GPNDATA_PAYMENT_DESCRIPTION');

        if(!isset($this->db)){
            $this->db = new Database(_DB_PREFIX_, _DB_SERVER_, _DB_NAME_, _DB_USER_, _DB_PASSWD_);
        }

		return parent::uninstall()
        && $this->unregisterHook('payment')
		&& $this->unregisterHook('paymentReturn')
		&& $this->unregisterHook('adminPayment')
		&& $this->unregisterHook('adminOrder')
		&& $this->unregisterHook('orderConfirmation')
		&& $this->unregisterHook('paymentOptions')
		&& $this->unregisterHook('displayPayment')
		&& $this->unregisterHook('displayPaymentTop');
	}

	public function getContent(){

		$html = '';
		if (Tools::isSubmit('submitConfigurationData')){
			if(isset($_POST['gpndata_pretty_name']) && empty($_POST['gpndata_pretty_name']) ||
			isset($_POST['gpndata_iframe_url']) && empty($_POST['gpndata_iframe_url']) ||
            isset($_POST['gpndata_payment_merchant_name']) && empty($_POST['gpndata_payment_merchant_name']) ||
            isset($_POST['gpndata_iframe_language_selected']) && empty($_POST['gpndata_iframe_language_selected']) ||
            isset($_POST['gpndata_payment_description']) && empty($_POST['gpndata_payment_description']) ||
            isset($_POST['gpndata_logo_url']) && empty($_POST['gpndata_logo_url'])
            ){
				$html = $this->l('Please fill the required fields. *');
			} else {

				Configuration::updateValue('GPNDATA_API_KEY', pSQL(Tools::getValue('gpndata_api_key')));
				Configuration::updateValue('GPNDATA_API_USER', pSQL(Tools::getValue('gpndata_api_user')));
				Configuration::updateValue('GPNDATA_API_PASSWORD', pSQL(Tools::getValue('gpndata_api_password')));
				Configuration::updateValue('GPNDATA_API_URL', pSQL(Tools::getValue('gpndata_api_url')));
				Configuration::updateValue('GPNDATA_PRETTY_NAME', pSQL(Tools::getValue('gpndata_pretty_name')));
				Configuration::updateValue('GPNDATA_IFRAME_URL', pSQL(Tools::getValue('gpndata_iframe_url')));
				Configuration::updateValue('GPNDATA_LOGO_URL', pSQL(Tools::getValue('gpndata_logo_url')));
                Configuration::updateValue('GPNDATA_PAYMENT_MERCHANT_NAME', pSQL(Tools::getValue('gpndata_payment_merchant_name')));
                Configuration::updateValue('GPN_DATA_IFRAME_LANGUAGE', pSQL(Tools::getValue('gpndata_iframe_language_selected')));
                Configuration::updateValue('GPNDATA_PAYMENT_DESCRIPTION', pSQL(Tools::getValue('gpndata_payment_description')));
                Configuration::updateValue('GPNDATA_PAYMENT_IFRAME_DESCRIPTION', pSQL(Tools::getValue('gpndata_payment_iframe_description')));
				$html = $this->l('Configuration updated successfully');
			}
		}

		$this->context->smarty->assign(array(
		'gpndata_form' => './index.php?tab=AdminModules&configure=gpndata&token='.Tools::getAdminTokenLite('AdminModules').'&tab_module='.$this->tab.'&module_name=gpndata',
		'gpndata_api_key' => Configuration::get('GPNDATA_API_KEY'),
		'gpndata_api_user' => Configuration::get('GPNDATA_API_USER'),
		'gpndata_api_password' => Configuration::get('GPNDATA_API_PASSWORD'),
		'gpndata_api_url' => Configuration::get('GPNDATA_API_URL'),
		'gpndata_pretty_name' => Configuration::get('GPNDATA_PRETTY_NAME'),
		'gpndata_iframe_url' => Configuration::get('GPNDATA_IFRAME_URL'),
		'gpndata_logo_url' => Configuration::get('GPNDATA_LOGO_URL'),
		'gpndata_notifications_url' => $this->context->link->getModuleLink('gpndata', 'ordervalidate', array(), Configuration::get('PS_SSL_ENABLED')),
		'gpndata_ssl' => Configuration::get('PS_SSL_ENABLED'),
		'gpndata_payment_merchant_name' => Configuration::get('GPNDATA_PAYMENT_MERCHANT_NAME'),
        'gpndata_iframe_language_selected' => Configuration::get('GPN_DATA_IFRAME_LANGUAGE'),
        'gpndata_payment_description' => Configuration::get('GPNDATA_PAYMENT_DESCRIPTION'),
        'gpndata_payment_iframe_description' => Configuration::get('GPNDATA_PAYMENT_IFRAME_DESCRIPTION'),
		'gpndata_confirmation' => $html));

		return $this->display(__FILE__, 'admin.tpl');
	}

    public function hookAdminOrder($params)
	{

	    
		if (!$this->active)
			return false;
		$order = new Order((int)$params['id_order']);
		if ($order->module != $this->name)
			return false;
		if (!Validate::isLoadedObject($order))
			return false;

        $transaction['error'] = '';
        $order_payment = OrderPayment::getByOrderId($params['id_order']);
        $transaction = count($order_payment) > 0 ? $this->db->getTransactionByMerchantTransactionId($order_payment[0]->transaction_id, true) : $this->db->getTransactionByOrderId($params['id_order'], true);
        if (!Configuration::get('PS_SSL_ENABLED') && !$this->must_use_ssl){
            $transaction['error'] = 'Please enable SSL on your server to use full features of this module.';
		}
			    
		if(count($transaction['transaction_adjustments']) > 0){
		    $last_transaction_state = $this->_API->CheckTransactionStatus($transaction['transaction']['merchant_trans_id'], $transaction['transaction']['gate_trans_id']);
            $transaction['transaction']['gateway_transaction_state'] = strval($last_transaction_state->transaction->state);
            $transaction['transaction']['gateway_transaction_description'] = strval($last_transaction_state->transaction->description);
		}
        $transaction['transaction']['hide_refund_button'] = isset($transaction['transaction']['payment_amount']) &&
       intval($transaction['transaction']['payment_amount']) === 0 ? 1 : 0;

        $transaction['transaction']['transaction_adjustments_count'] = count($transaction['transaction_adjustments']);
		$this->context->smarty->assign(
            array(
		        'gpndata_form' => './index.php?tab=AdminOrders&id_order='.(int)$order->id.'&vieworder&token='.Tools::getAdminTokenLite('AdminOrders'),
		        'transaction' => $transaction,
                'order_id' =>  $params['id_order']
                ));

		return $this->display(__FILE__, 'admin-order.tpl');
	}

    public function hookdisplayPaymentTop()
    {
        if (!$this->active || !Tools::getValue('gpnerror')) {
            return;
        }

        $errorMessage = Tools::getValue('gpnerror');

        $this->context->smarty->assign(array(
            'errorMessage' => $errorMessage,
            'module' => "gpndata"
        ));

        return $this->display(__FILE__, 'error.tpl');
    }

    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }

        $order = empty($params['objOrder']) ? $params['order'] : $params['objOrder'];

		if ($order->module != $this->name)
			return false;

		if ($order && Validate::isLoadedObject($order) && isset($order->valid)){
            $this->db->updateTransactionByMerchantTransactionId(
                array('merchant_trans_id' => base64_decode(Tools::getValue('trans')),
                      'order_id' => Tools::getValue('id_order')
            ));

            if ($params['objOrder'] && Validate::isLoadedObject($params['objOrder']) && isset($params['objOrder']->valid)){
                if (version_compare(_PS_VERSION_, '1.5', '>=') && isset($params['objOrder']->reference))
                    $this->smarty->assign('gpndata_order',
                        array('id' => $params['objOrder']->id, 'reference' => $params['objOrder']->reference, 'current_state' => $params['objOrder']->current_state));
                else
                    $this->smarty->assign('gpndata_order', array('id' => $params['objOrder']->id, 'current_state' => $params['objOrder']->current_state));

                return $this->display(__FILE__, 'order-confirmation-gpn.tpl');
            }
		}
    }

    public function hookPaymentOptions($params){

        $payments_option = new PaymentOption();
        $payments_option->setCallToActionText($this->l(Configuration::get('GPNDATA_PAYMENT_DESCRIPTION')))
                        ->setAction($this->context->link->getModuleLink($this->name, 'payment'))
                        ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/img/logo-s.jpg'));

        $payments_options = [$payments_option];
        return $payments_options;
    }

	public function hookPayment($params){
		if (!$this->active){
			return false;
		}

		if(Configuration::get('GPNDATA_PRETTY_NAME')  == '' || Configuration::get('GPNDATA_IFRAME_URL')  == ''){
			return false;
		}
        $this->context->smarty->assign(array('path_gpn' => $this->_path));
        return $this->display(__FILE__, 'payment.tpl');
    }

    private function redirectError($returnMessage){
		Tools::redirect(
				$this->context->link->getPageLink(
						'order',
						true,
						null,
						[
								'step'     => '3',
								'gpnerror' => $returnMessage
						]
				)
		);
		die();
	}
  public function validateOrderByCartId()
	{
		/*
        if(!isset($this->db)){
            $this->_include();
            $this->_load();
        }
		$cart = new Cart(48218);
		ini_set('display_errors', 1);
		ini_set('display_startup_errors', 1);
		error_reporting(E_ALL);
		$this->validateOrder((int)$cart->id, 2, (float)'126.50', $this->displayName, null, array(), null, false, $cart->secure_key);
		die('done');*/
	}
    public function setOrderStatusAndValidate($params)
	{
        if(!isset($this->db)){
            $this->_include();
            $this->_load();
        }

		$cart = $this->context->cart;
		$new_status =  _PS_OS_ERROR_;
        $valid_key = sha1(sha1($params['transactionid']).'::'.Configuration::get('GPNDATA_PRETTY_NAME'));
        if($valid_key != $params['key']){
            $this->redirectError('Technical error occurred. Please contact web shop owner. Payment failed.');
        }
        switch ($params['status']){
            case TransactionStatus::SUCCESS:
                $new_status = OrderStatus::PAYMENT_ACCEPTED; //by default statuses prestashop Payment Success
                break;
            case TransactionStatus::DECLINED:
                $new_status = GpnDataOrderStatus::getOrderStateDeclinedId();
                break;
        	default:
                $new_status = OrderStatus::PAYMENT_ERROR; //by default statuses prestashop Payment error
                break;
        }

        $merchant_trans_id = $params['transactionid'];
        $encoded_transaction = base64_encode($params['transactionid']);
        $id_cart = Tools::getValue('id_cart');
        $cart = new Cart($id_cart);
		if(Validate::isLoadedObject($cart)){
			$total = (float)($cart->getOrderTotal(true, Cart::BOTH));
            $order_id = Order::getOrderByCartId($id_cart);
            $order = new Order($order_id);
            $order_validated = empty(Validate::isLoadedObject($order)) ? false : true;
            if(!$order_validated){
                $this->validateOrder((int)$cart->id, (int)$new_status, (float)$total, $this->displayName, null, array(), null, false, $cart->secure_key);
                $order_id = (int)$this->currentOrder;
            }
            $transaction_exits = $this->db->transactionExistsByMerchantTransactionId($merchant_trans_id);
            if(!$transaction_exits){
                $this->db->addTransactionIfDoesNotExist(array
                (
                       'order_id' => $order_id,
                       'merchant_trans_id' => $merchant_trans_id,
                       'curcode' =>  $params['currency'],
                       'payment_amount' => $total,
                       'transaction_ref' => $params['ref'],
                       'is_payment_failed' => ($new_status == _PS_OS_ERROR_ ? 1 : 0),
                       'is_declined' => ($new_status === GpnDataOrderStatus::getOrderStateDeclinedId() ? 1 : 0)
                ));
            }
            else{
                $this->db->updateTransactionByMerchantTransactionId(array('merchant_trans_id' => $merchant_trans_id,
                    'order_id' => $order_id,
                    'is_declined' => ($new_status === GpnDataOrderStatus::getOrderStateDeclinedId() ? 1 : 0)));
            }

			$this->addTransactionId($order_id, $params['transactionid']);
            if (_PS_VERSION_ < 1.5)
                $redirect = __PS_BASE_URI__.'order-confirmation.php?id_cart='.(int)$this->context->cart->id.'&id_module='.(int)$this->id.'&id_order='.(int)$order_id.'&key='.$this->context->customer->secure_key.'&trans='.$encoded_transaction;
            else
            {
                if($new_status == _PS_OS_PAYMENT_){
                    $redirect = __PS_BASE_URI__.'index.php?controller=order-confirmation&id_cart='.(int)$this->context->cart->id.'&id_module='.(int)$this->id.'&id_order='.(int)$order_id.'&key='.$this->context->customer->secure_key.'&trans='.$encoded_transaction;
                }
                else{
                    $redirect = __PS_BASE_URI__.'index.php?controller=order-detail&id_order='.(int)$order_id;
                }
            }

			header('Location: '.$redirect);
			exit;
		}
	}

    public function handleNotification($notification_xml){
		$message = 'OK';
		$xml_data = simplexml_load_string(stripslashes(urldecode($notification_xml)));

		if (empty($xml_data)){
			unset($xml_data);
			$resstr = '
			    <?xml version="1.0" encoding="utf-8" ?>
			    <transaction>
			        <result>ERROR</result >
			        <gatetransid></gatetransid>
			        <merchanttransid></merchanttransid>
			        <errorcode></errorcode>
			        <errormessage>No XML Data</errormessage>
			    </transaction>';
			echo $resstr;
			die();
		}

        $this->_include();
		$this->_load();

		If (!$this->_API->validateNotificationChecksum($xml_data)){
			unset($xml_data);
			$resstr = '
			    <?xml version="1.0" encoding="utf-8" ?>
			        <transaction>
			        <result>ERROR</result >
			        <gatetransid></gatetransid>
			        <merchanttransid></merchanttransid>
			        <errorcode></errorcode>
			        <errormessage>Invalid Checksum</errormessage>
			    </transaction>';
			echo $resstr;
			die();
		}

        $reference = strval($xml_data->transref);
        $transaction_status = strval($xml_data->status);
        $transaction_state = isset($xml_data->state) ? strval($xml_data->state) : '';
        $transaction_merchanttransid = strval($xml_data->merchanttransid);
        $transaction_gatetransid = strval($xml_data->gatetransid);
        $transaction_amount = strval($xml_data->amount);
        $transaction_curcode = strval($xml_data->curcode);
        $transaction_description = strval($xml_data->description);
        $installment_count = isset($xml_data->installment) ? intval($xml_data->installment) : 0;
        switch (intval($xml_data->apiCmd)) {
            case 850:
				$message = $this->validateOrderFromNotification($transaction_merchanttransid, $transaction_status, $transaction_state, $transaction_gatetransid, $transaction_amount,
				$transaction_description, $transaction_curcode, $reference, 850, $installment_count);
				break;
            case 860:
				$message = $this->validateOrderFromNotification($transaction_merchanttransid, $transaction_status, $transaction_state, $transaction_gatetransid, $transaction_amount,
				$transaction_description, $transaction_curcode, $reference, 860, $installment_count);
				break;
            case 880: //This is Chargeback Notification
                $chgbk_chbDate = strval($xml_data->disputeDate);
                $chgbk_reasonCode = strval($xml_data->reasonCode);
                $chgbk_reasonText= strval($xml_data->reasonText);
                $message = $this->validateOrderFromNotification($transaction_merchanttransid, $transaction_status, $transaction_state, $transaction_gatetransid, $transaction_amount,
                $chgbk_reasonText.'. Reason code: '.$chgbk_reasonCode.'\n ARN:'.strval($xml_data->arn).'\n Dispute date: '.$chgbk_chbDate, $transaction_curcode, $reference, 880, 0);

                break;
            default:
                unset($xml_data);
		}

        $this->_notificationRespond($message, $transaction_gatetransid);
        unset($xml_data);
	}

    public function _notificationRespond($message, $gateway_transaction_id){
        $msg = $this->return_notification_error ? 'ERROR' : 'OK';
        $resstr = '
			<?xml version="1.0" encoding="utf-8" ?>
			<transaction>
				<result>'.$msg.'</result>
				<plugin-message>'.$message.'</plugin-message>
				<gatetransid>'.$gateway_transaction_id.'</gatetransid>
                <prestashop-version>'._PS_VERSION_.'</prestashop-version>
			</transaction>';
        echo $resstr;
    }

    private $return_notification_error = false;
	public function validateOrderFromNotification($merchanttransid, $status, $state, $transaction_gatetransid, $amount,
        $description, $curcode, $reference, $type, $installment_count){
		$val = explode('-', $merchanttransid);
		$id_cart = $val[0];
        $order_id = Order::getOrderByCartId($id_cart);

		$order = new Order($order_id);
        $order_validated = empty(Validate::isLoadedObject($order)) ? false : true;
        $message = "Order validation status: $order_validated. ";
        $transaction_exists = $this->db->transactionExistsByMerchantTransactionId($merchanttransid);
        if(!$order_validated || !$transaction_exists){
            if(!$transaction_exists){
                $transaction = array
                (
                    'order_id' => $order_id,
                    'transaction_ref' => $reference,
                    'gate_trans_id' => $transaction_gatetransid,
                    'merchant_trans_id' => $merchanttransid,
                    'curcode' =>  $curcode,
                    'payment_amount' => $amount,
                    'installment_count' => $installment_count,
                    'is_captured' => ($status == TransactionStatus::CAPTURED) ? 1 : 0,
                    'is_authorized' => ($status == TransactionStatus::AUTHORIZED) ? 1 : 0,
                    'is_declined' => ($status == TransactionStatus::DECLINED) ? 1 : 0
                );
                $this->db->addTransaction($transaction);
                $message = $message. ' Payment data was inserted to transactions table.';
                //dont use notification first notification for validate order
                $order_validated = true;
            }
            else{
                $transaction = $this->db->getTransaction($transaction_gatetransid);
                if(!empty($transaction['order_id'])){
                    $order_validated = intval($transaction['order_id']) > 0 ? true : false;
                }
            }
        }
        else{
            $transaction = $this->db->getTransaction($transaction_gatetransid);
            if(intval($transaction['is_declined']) != 1 && intval($transaction['is_captured']) == 1){
                $last_status_exist = false;
                $refund_amount = null;
                if($type == NotificationResponses::RefundNotification){
                    $last_status_exist =  $this->db->getLastTransactionAdjustment($transaction_gatetransid) == $status;
                    if(!$last_status_exist)
                        $refund_amount = (floatval($transaction['payment_amount']) - floatval($amount));
                }
                if(!$last_status_exist){
                    $this->db->addTransactionAdjustmentFromArray(
                    array(
                          'gate_trans_id' => $transaction_gatetransid,
                          'amount' => $amount,
                          'status' => $status,
                          'state' => $state,
                          'description' => $description
                          ));
                    $message = $message. ' Transaction adjustment was added.';
                }
            }
            $message = $message .  $this->updateTransactionStatus($transaction_gatetransid, $status, $transaction, $order_id, $type);
        }

		if(!$order_validated && ($state == TransactionStatus::CAPTURED || $state == TransactionStatus::DECLINED)){
			$cart = new Cart($id_cart);
			if(Validate::isLoadedObject($cart)){
                if($state == TransactionStatus::DECLINED){
                    $current_state = GpnDataOrderStatus::getOrderStateDeclinedId();
                }
                else{
                    $current_state = $state == TransactionStatus::CAPTURED ? _PS_OS_PAYMENT_ : _PS_OS_ERROR_;
                }
                $order_id = Order::getOrderByCartId($id_cart);
                $order = new Order($order_id);
                $order_validated = empty(Validate::isLoadedObject($order)) ? false : true;
                if(!$order_validated){
                    if(empty($transaction['date_modified'])){
                        $this->return_notification_error = true;
                        $message = $message . " Return notification error due to order validation on FrontEnd. Upon next notification arival order will be validated if not validated on FrontEnd.";
                        $order_id = 0;
                        goto UPDATE_TRANS_STATUS;
                    }
                    else{
                        $minutes_difference = intval(abs(strtotime($transaction['date_added']) - time()) / 60);
                        $message = $message . "Time transaction added: " .  $transaction['date_added'] . " Allow order validate after 4 minutes, current value: ".intval(abs(strtotime($transaction['date_added']) - time()) / 60);
                        if(intval($transaction['is_notification_pending']) == 1 && $minutes_difference >= 4){
                            $message = $message . "Try to validate order";
                            $this->validateOrder((int)$cart->id, (int)$current_state, (float)$amount, $this->displayName, null, array(), null, false, $cart->secure_key);
                            $message = $message . "Order was successfully created with status $current_state. Status 8 = declined, status 2 approved!";
                            $this->db->updateTransactionByMerchantTransactionId(array('merchant_trans_id' => $merchanttransid, 'is_notification_pending' => 0));
                        }
                        else{
                            $this->return_notification_error = true;
                            $message = $message . " Return notification error status due to need to wait time to handle order validation in BackEnd. Total minutes elapesed: ".$minutes_difference;
                        }
                    }
                    $order_id = (int)$this->currentOrder;
                }

                if(empty($transaction['order_id']) && $this->currentOrder != null){

                    UPDATE_TRANS_STATUS:
                    $this->db->updateTransactionByMerchantTransactionId(array('merchant_trans_id' => $merchanttransid, 'order_id' => $order_id, 'is_captured' => ($status == TransactionStatus::CAPTURED ? 1: 0),
                        'is_declined' => ($status == TransactionStatus::DECLINED ? 1: 0)));
                }
                if (version_compare(_PS_VERSION_, '1.5', '>=') && $this->currentOrder != null){
                    $new_order = new Order((int)$this->currentOrder);
                    if (Validate::isLoadedObject($new_order)){
                        $this->addTransactionId($this->currentOrder, $merchanttransid);
                    }
                }
			}
			else {
				$message = $message .'Error - Invalid cart Id - merchant transaction id. This message represents error!';
			}
		}
        if($type === NotificationResponses::DisputeNotification){
           if(GpnDataOrderStatus::setOrderNewStatus($order_id, GpnDataOrderStatus::getOrderStateChargebackId())){
               $message = $message. 'Updated PS Order status to: ' . GpnDataOrderStatus::$PARTIAL_CAPTURED['value'];
           }
        }
		return $message;
	}

    private function updateTransactionStatus($transaction_gatetransid, $status, $transaction, $order_id, $type){

        $update_result =  $this->db->updateTransaction(
                           array(
                                 'gate_trans_id' => $transaction_gatetransid,
                                 'is_captured' => ((intval($transaction['is_captured']) == 1 ? $transaction['is_captured'] : ((intval($transaction['is_captured']) == 0 && $status == TransactionStatus::CAPTURED) ? 1 : 0))),
                                 'is_authorized' => (intval($transaction['is_authorized']) == 1 && $status == TransactionStatus::AUTHORIZED) ? 1 : 0,
                                 'is_voided' => (intval($transaction['is_voided']) == 1 ? 1 : (intval($transaction['is_voided']) == 0 && $status == TransactionStatus::CANCELED) ? 1 : 0),
                                 'is_refunded' => (intval($transaction['is_refunded']) == 1 ? 1 : (intval($transaction['is_refunded']) == 0 && $status == TransactionStatus::REFUNDED) ? 1 : 0),
                                 'is_refund_pending' => ($status == TransactionStatus::REFUND_PENDING) ? 1 : 0,
                                 'is_chargeback' => ($type == NotificationResponses::DisputeNotification ? 1 : 0),
                                 'is_declined' => ($status == TransactionStatus::DECLINED ? 1 : 0),
                                 'payment_amount' => empty($refund_amount) ? $transaction['payment_amount'] : $refund_amount,
                                 'order_id' => $order_id
                                 ));
        return " Transactions table status update result: $update_result.";
    }

	public function addTransactionId($id_order, $id_transaction){
		if (version_compare(_PS_VERSION_, '1.5', '>='))
		{
			$new_order = new Order((int)$id_order);
			if (Validate::isLoadedObject($new_order))
			{
				$payment = $new_order->getOrderPaymentCollection();
				if (isset($payment[0])){
					$payment[0]->transaction_id = pSQL($id_transaction);
					$payment[0]->save();
				}
			}
		}
	}
}
